#include <sys/mman.h>
#include <fcntl.h>  

#define GPIO0_ADDR 0x44E07000
#define GPIO1_ADDR 0x4804C000
#define GPIO2_ADDR 0x481AC000
#define GPIO3_ADDR 0x481AE000

#define GPIO_SIZE  0x00000FFF

// OE: 0 is output, 1 is input
#define GPIO_OE  0x14d
#define GPIO_IN  0x14e
#define GPIO_OUT 0x14f


//redefine for user layer
#define GPIO_COUNT 22


volatile unsigned char lld_addr[]={1    ,1    ,1    ,2    ,1    ,0    ,1    ,0    ,0    ,2    ,2    ,2    ,2    ,1     ,0    ,1    ,2    ,1    ,2    ,3    ,3    ,3};
volatile unsigned   int lld_ios[]={1<<28,1<<18,1<<19,1<< 5,1<<13,1<<23,1<<15,1<<27,1<<22,1<<22,1<<23,1<< 3,1<< 4,1<<12 ,1<<26,1<<14,1<< 1,1<<29,1<<24,1<<19,1<<17,1<<14};

#define GPIO_TYPE_OUT 0
#define GPIO_TYPE_IN  1
#define GPIO_STATUS_LOW  0
#define GPIO_STATUS_HIGH 1
#define GPIO_STATUS_GET  2


#define GPOSYS1  0
#define GPOSYS2  1
#define GPOSYS3  2
#define GPO01    3
#define GPO02    4
#define GPO03    5
#define GPO04    6
#define GPO05    7
#define GPO06    8
#define GPO07    9
#define GPO08    10
#define GPI01    11
#define GPI02    12
#define GPI03    13
#define GPI04    14
#define GPI05    15
#define GPI06    16
#define GPI07    17
#define GPI08    18
#define GPISW1   19
#define GPISW2   20
#define GPOBUZ   21

// I/O access
volatile unsigned * lld_gpio[4];
static void GPIO_TYPE(int io , int type)
{
       if(io >= GPIO_COUNT || io < 0 ){return ;}
       
       *(lld_gpio[lld_addr[io]] + GPIO_OUT) = *(lld_gpio[lld_addr[io]] + GPIO_OUT) & (~lld_ios[io]);
       
       

       if(type == GPIO_TYPE_OUT)
       {

           *(lld_gpio[lld_addr[io]] + GPIO_OE) = *(lld_gpio[lld_addr[io]]  + GPIO_OE) & (~lld_ios[io]);
                     
       }
       else if(type == GPIO_TYPE_IN)
       {
           *(lld_gpio[lld_addr[io]] + GPIO_OE) = *(lld_gpio[lld_addr[io]]  + GPIO_OE) | lld_ios[io];
            
       }
       
       
}

static int GPIO_STATUS(int io , int cmd)
{
  if(io >= GPIO_COUNT || io < 0 ){return -1;}
       
 if(cmd == GPIO_STATUS_LOW)
 {
   *(lld_gpio[lld_addr[io]] + GPIO_OUT) = *(lld_gpio[lld_addr[io]]  + GPIO_OUT) & (~lld_ios[io]);
         
   return 0;
        
 }
 else if(cmd == GPIO_STATUS_HIGH)
 {
   *(lld_gpio[lld_addr[io]]+ GPIO_OUT) = *(lld_gpio[lld_addr[io]]+ GPIO_OUT) | lld_ios[io];
      
   return 1;
 }
 else if(cmd == GPIO_STATUS_GET)
 {  
      if(*(lld_gpio[lld_addr[io]] + GPIO_OE) & lld_ios[io])
      {
        return (*(lld_gpio[lld_addr[io]] + GPIO_IN) & lld_ios[io])?1:0;
        
      }
      else
      {
        return (*(lld_gpio[lld_addr[io]] + GPIO_OUT) & lld_ios[io])?1:0;
      } 
 }
 return -1;
}




static void GPIO_LLD004_TYPE_INIT()
{
 int i;
 GPIO_TYPE(GPOSYS1 , GPIO_TYPE_OUT);
 GPIO_TYPE(GPOSYS2 , GPIO_TYPE_OUT);
 GPIO_TYPE(GPOSYS3 , GPIO_TYPE_OUT);
 
 
 for(i=0;i<8;i++)
 {
 GPIO_TYPE(GPO01+i , GPIO_TYPE_OUT);
 GPIO_TYPE(GPI01+i , GPIO_TYPE_IN);
 }
 
 
 GPIO_TYPE(GPISW1 , GPIO_TYPE_IN);
 GPIO_TYPE(GPISW2 , GPIO_TYPE_IN);
 GPIO_TYPE(GPOBUZ , GPIO_TYPE_OUT);

}

static void GPIO_LOAD_API()
{
int mem_fd;
    
    /* open /dev/mem */
    if ((mem_fd = open("/dev/mem", O_RDWR|O_SYNC) ) < 0) {
            printf("can't open /dev/mem \n");
            exit (-1);
    }
 


    // Always use the volatile pointer!
    lld_gpio[0] = (volatile unsigned *)mmap(0,GPIO_SIZE,PROT_READ|PROT_WRITE,MAP_SHARED,mem_fd,GPIO0_ADDR);
    lld_gpio[1] = (volatile unsigned *)mmap(0,GPIO_SIZE,PROT_READ|PROT_WRITE,MAP_SHARED,mem_fd,GPIO1_ADDR);
    lld_gpio[2] = (volatile unsigned *)mmap(0,GPIO_SIZE,PROT_READ|PROT_WRITE,MAP_SHARED,mem_fd,GPIO2_ADDR);
    lld_gpio[3] = (volatile unsigned *)mmap(0,GPIO_SIZE,PROT_READ|PROT_WRITE,MAP_SHARED,mem_fd,GPIO3_ADDR);
 
 
 close(mem_fd);

//gpio2 gpio3  
system("echo 67 > /sys/class/gpio/export");
system("echo 115 > /sys/class/gpio/export");

GPIO_LLD004_TYPE_INIT();

}
